==> react-native-ctp-odp-1.6.3.tgz: package/android/src/main/java/com/ctpodp/PathUtilities.java <==
package com.ctpodp;

import android.graphics.PointF;
import android.util.Log;

import com.ctpodp.objects.ProcessedInstruction;
import com.ctpodp.objects.fromReact.Geojson.GeoJson;
import com.ctpodp.objects.fromReact.Geojson.GeoJsonGeometry;
import com.ctpodp.objects.fromReact.Geojson.GeojsonPathObj;
import com.ctpodp.objects.fromReact.Geojson.GeojsonPathWaypointObj;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.jibestream.jmapandroidsdk.astar.ASNode;
import com.jibestream.jmapandroidsdk.astar.PathPerFloor;
import com.jibestream.jmapandroidsdk.components.ActiveVenue;
import com.jibestream.jmapandroidsdk.components.Associations;
import com.jibestream.jmapandroidsdk.components.Destination;
import com.jibestream.jmapandroidsdk.components.Waypoint;
import com.jibestream.jmapandroidsdk.jcontroller.JController;
import com.jibestream.jmapandroidsdk.main.PointD;
import com.jibestream.jmapandroidsdk.main.Utilities;
import com.jibestream.jmapandroidsdk.rendering_engine.drawables.JShapeDrawable;
import com.jibestream.navigationkit.instructionfactory.Instruction;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

public class PathUtilities {

  //{
  //                distance: <total distance in meters>,
  //                duration: <total duration in seconds>,
  //                startTime: <unix timestamp in milliseconds>,
  //                endTime: <unix timestamp in milliseconds>,
  //                legGeometry: <GeoJson>,
  //                steps: <waypoints> {
  //                                distance: <distance in meters for this step/waypoint>,
  //                                directions: <actual turn-by-turn instruction, in feet>,
  //                                lat: <latitude>,
  //                                lon: <longitude>,
  //                                direction: <direction change direction, ex. "Left", "Forward", ...>,
  //                                prepend: <direction change type, ex. "Turn", "", ...>
  //                }
  //}

  private static double getDistance(Double[] a, Double[] b) {
    return Math.sqrt(Math.pow(a[0] - b[0], 2) + Math.pow(a[1] - b[1], 2));
  }

  public static PointF convertLocalToLatLng(JController controller, float x, float y) {
    PointD convertPoint = new PointD(x, y);
    Utilities.convertLocalToEpsg4326(convertPoint, controller.getActiveVenue());
    return new PointF((float) convertPoint.y, (float) convertPoint.x);
  }

  public static PointF convertLatLngToLocal(JController controller, float lat, float lng) {
    PointD convertPoint = new PointD(lng, lat);
    Utilities.convertEpsg4326ToLocal(convertPoint, controller.getActiveVenue());
    return new PointF((float) convertPoint.x, (float) convertPoint.y);
  }

  public static ProcessedInstruction[] processInstructions(Instruction[] instructions, ActiveVenue venue, String unit, float mmPerPixel) {
    List<ProcessedInstruction> processedInstructions = new ArrayList<>();
    for (int i = 0; i < instructions.length; i++) {

      Instruction instruction = instructions[i];
      ProcessedInstruction processedInstruction = new ProcessedInstruction();
      if (instruction.getDecisionPoint() != null) {
        processedInstruction.wp = instruction.getDecisionPoint().id;
        processedInstruction.cornerLocation = instruction.getDecisionPoint().getCoordinates();
      } else {
        processedInstruction.wp = instruction.getCompletionPoint().id;
        processedInstruction.cornerLocation = instruction.getCompletionPoint().getCoordinates();
      }
      Log.d("DISTANCE", instruction.getText() + ", " + instruction.getPixelDistance());
      Float distanceMeters = instruction.getPixelDistance() * mmPerPixel / 1000;
      if (distanceMeters == 0 && processedInstructions.size() != 0) {
        distanceMeters = (float) getDistance(processedInstructions.get(processedInstructions.size() - 1).cornerLocation, processedInstruction.cornerLocation) * mmPerPixel / 1000;
      }
      Float distance = distanceMeters;
      String units = "";

      switch(unit) {
        case "FEET":
          distance *= 3.28084F;
          units = Math.ceil(distance) == 1 ? " foot" : " feet";
          break;
        case "METERS":
        default:
          units = Math.ceil(distance) == 1 ? " meter" : " meters";
          break;
      }
      processedInstruction.distance = Double.valueOf(distanceMeters);
      processedInstruction.remainingDistance = Double.valueOf(distanceMeters);
      processedInstruction.direction = instruction.getDirection();
      if (instruction.getText().startsWith("Arrive")) {
        processedInstruction.direction = "Arrive";
      }

      String prepend = "";
      if (instruction.getDirection().equals("Left") || instruction.getDirection().equals("Right")) {
        prepend = "Turn ";
      }
      processedInstruction.prepend = prepend;

      if (instruction.getSurroundingElements() != null &&
        instruction.getSurroundingElements().getShapes() != null &&
        instruction.getSurroundingElements().getShapes().length > 0) {
        JShapeDrawable shape = instruction.getSurroundingElements().getShapes()[0].getShape();

        //Get data-unit meta data which is associated to units
        Object[] dataUnits = shape.getMetaData("data-unit");

        //Make sure shape has destination association
        if (dataUnits != null && dataUnits.length > 0) {
          int destinationId = (int) dataUnits[0];

          //Get destination object from Id
          Destination destination = venue.getDestinations().getById(destinationId);

          //Get destination name
          String destinationName = destination.getName();

          if (destinationName != null) {
            processedInstruction.destinationName = destinationName;

            processedInstruction.dest = destination.id;
            processedInstruction.fullText = instruction.getText() + " at " + destinationName + " in " + (int)Math.ceil(distance) + units;

          } else {
            processedInstruction.dest = destination.id;
            processedInstruction.fullText = instruction.getText() + " in " + (int)Math.ceil(distance) + units;
          }
        } else {
          processedInstruction.destinationName = "";
          processedInstruction.dest = null;
          processedInstruction.fullText = instruction.getText() + " in " + (int)Math.ceil(distance) + units;
        }
      } else {
        processedInstruction.destinationName = "";
        processedInstruction.dest = null;
        processedInstruction.fullText = instruction.getText() + " in " + (int)Math.ceil(distance) + units;
      }
      processedInstructions.add(processedInstruction);
    }
    return processedInstructions.toArray(new ProcessedInstruction[processedInstructions.size()]);
  }

  public static WritableArray getMapFromPath(JController controller, PathPerFloor[] pathPerFloors) {
    WritableArray pathArr = ReactUtilities.classToArray(pathPerFloors);
    WritableArray pathArrW = Arguments.createArray();
    for (int i = 0; i < pathArr.size(); i++) {
      ReadableMap path = pathArr.getMap(i);

      WritableMap pathW = Arguments.createMap();
      pathW.putInt("mapId", path.getInt("mapId"));
      pathW.putMap("mover", path.getMap("mover"));
      WritableArray pointsW = Arguments.createArray();
      pathW.putDouble("cost", path.getDouble("cost"));
      pathW.putDouble("pixelDistance", path.getDouble("pixelDistance"));
      pathW.putDouble("mmDistance", path.getDouble("mmDistance"));

      for (int j = 0; j < path.getArray("points").size(); j++) {

        ReadableMap point = path.getArray("points").getMap(j);
        WritableMap pointW = Arguments.createMap();

        pointW.putInt("id", point.getInt("id"));
        pointW.putInt("mapId", point.getInt("mapId"));
        pointW.putDouble("x", point.getDouble("x"));
        pointW.putDouble("y", point.getDouble("y"));
        pointW.putDouble("z", point.getDouble("z"));

        PointF latLng = convertLocalToLatLng(controller, (float)point.getDouble("x"), (float)point.getDouble("y"));
        pointW.putDouble("latitude", latLng.x);
        pointW.putDouble("longitude", latLng.y);

        int id = point.getInt("id");
        Waypoint waypoint = Arrays.stream(controller.getActiveVenue().getMaps().getAllWaypoints()).filter(wp -> wp.id == id).findFirst().get();
        Associations associations = waypoint.getAssociations();
        WritableMap assocaitionsMap = ReactUtilities.classToMap(associations);

        pointW.putMap("associations", assocaitionsMap);
        pointsW.pushMap(pointW);
      }

      pathW.putArray("points", pointsW);
      pathArrW.pushMap(pathW);
    }
    return pathArrW;
  }
}

==> react-native-ctp-odp-1.6.4.tgz: package/android/src/main/java/com/ctpodp/PathUtilities.java <==
package com.ctpodp;

import android.graphics.PointF;
import android.util.Log;

import com.ctpodp.objects.ProcessedInstruction;
import com.ctpodp.objects.fromReact.Geojson.GeoJson;
import com.ctpodp.objects.fromReact.Geojson.GeoJsonGeometry;
import com.ctpodp.objects.fromReact.Geojson.GeojsonPathObj;
import com.ctpodp.objects.fromReact.Geojson.GeojsonPathWaypointObj;
import com.facebook.react.bridge.Arguments;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.bridge.WritableArray;
import com.facebook.react.bridge.WritableMap;
import com.jibestream.jmapandroidsdk.astar.ASNode;
import com.jibestream.jmapandroidsdk.astar.PathPerFloor;
import com.jibestream.jmapandroidsdk.components.ActiveVenue;
import com.jibestream.jmapandroidsdk.components.Associations;
import com.jibestream.jmapandroidsdk.components.Destination;
import com.jibestream.jmapandroidsdk.components.Waypoint;
import com.jibestream.jmapandroidsdk.jcontroller.JController;
import com.jibestream.jmapandroidsdk.main.PointD;
import com.jibestream.jmapandroidsdk.main.Utilities;
import com.jibestream.jmapandroidsdk.rendering_engine.drawables.JShapeDrawable;
import com.jibestream.navigationkit.instructionfactory.Instruction;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Calendar;
import java.util.List;

public class PathUtilities {

  //{
  //                distance: <total distance in meters>,
  //                duration: <total duration in seconds>,
  //                startTime: <unix timestamp in milliseconds>,
  //                endTime: <unix timestamp in milliseconds>,
  //                legGeometry: <GeoJson>,
  //                steps: <waypoints> {
  //                                distance: <distance in meters for this step/waypoint>,
  //                                directions: <actual turn-by-turn instruction, in feet>,
  //                                lat: <latitude>,
  //                                lon: <longitude>,
  //                                direction: <direction change direction, ex. "Left", "Forward", ...>,
  //                                prepend: <direction change type, ex. "Turn", "", ...>
  //                }
  //}

  private static double getDistance(Double[] a, Double[] b) {
    return Math.sqrt(Math.pow(a[0] - b[0], 2) + Math.pow(a[1] - b[1], 2));
  }

  private static double getAngle(Double[] start, Double[] corner, Double[] end) {
    Double initialAngle = Math.atan((start[1] - corner[1])/ (start[0] - corner[0]));
    Double finalAngle = Math.atan((corner[1] - end[1])/ (corner[0] - end[0]));
    Double angleDelta = (finalAngle - initialAngle) % 360;
    if (angleDelta > 180) {
      angleDelta -= 360;
    }
    else if (angleDelta < -180){
      angleDelta += 360;
    }
    return angleDelta;
  }

  public static PointF convertLocalToLatLng(JController controller, float x, float y) {
    PointD convertPoint = new PointD(x, y);
    Utilities.convertLocalToEpsg4326(convertPoint, controller.getActiveVenue());
    return new PointF((float) convertPoint.y, (float) convertPoint.x);
  }

  public static PointF convertLatLngToLocal(JController controller, float lat, float lng) {
    PointD convertPoint = new PointD(lng, lat);
    Utilities.convertEpsg4326ToLocal(convertPoint, controller.getActiveVenue());
    return new PointF((float) convertPoint.x, (float) convertPoint.y);
  }

  public static Double[] getCoordsFromInstruction(Instruction instruction) {
    if (instruction.getDecisionPoint() != null) {
      return instruction.getDecisionPoint().getCoordinates();
    } else {
      return instruction.getCompletionPoint().getCoordinates();
    }
  }

  public static ProcessedInstruction[] processInstructions(Instruction[] instructions, ActiveVenue venue, String unit, float mmPerPixel) {

    List<Instruction> filteredInstructions = new ArrayList<Instruction>();
    for (int i = instructions.length - 1; i >= 0; i--) {
      Instruction instruction = instructions[i];
      if (filteredInstructions.size() == 0) {
        filteredInstructions.add(0, instruction);
        continue;
      }
      else if (i == 0) {
        filteredInstructions.add(0, instruction);
        continue;
      }
      // guaranteed to not be i = 0 || i = length - 1
      Instruction prevInstruction = filteredInstructions.get(0);
      double angleDiff = getAngle(
        getCoordsFromInstruction(prevInstruction),
          getCoordsFromInstruction(instruction),
            getCoordsFromInstruction(instructions[i-1]));
      double ANGLE_THRESHOLD = 2 * Math.PI / 180; // degrees
      if (Math.abs(angleDiff) < ANGLE_THRESHOLD) {
        // Angle is less than threshold, dont include in final
        prevInstruction.setOriginPoint(instruction.getOriginPoint());
        prevInstruction.setPixelDistance(instruction.getPixelDistance() + prevInstruction.getPixelDistance());
        filteredInstructions.remove(0);
        filteredInstructions.add(0, prevInstruction);
      }
      else {
        filteredInstructions.add(0, instruction);
      }
    }

    List<ProcessedInstruction> processedInstructions = new ArrayList<>();
    for (int i = 0; i < filteredInstructions.size(); i++) {

      Instruction instruction = filteredInstructions.get(i);
      ProcessedInstruction processedInstruction = new ProcessedInstruction();
      instruction.getOriginPoint();
        instruction.getCompletionPoint();
      if (instruction.getDecisionPoint() != null) {
        processedInstruction.wp = instruction.getDecisionPoint().id;
        processedInstruction.cornerLocation = instruction.getDecisionPoint().getCoordinates();
      } else {
        processedInstruction.wp = instruction.getCompletionPoint().id;
        processedInstruction.cornerLocation = instruction.getCompletionPoint().getCoordinates();
      }
      Log.d("DISTANCE", instruction.getText() + ", " + instruction.getPixelDistance());
      Float distanceMeters = instruction.getPixelDistance() * mmPerPixel / 1000;
      if (distanceMeters == 0 && processedInstructions.size() != 0) {
        distanceMeters = (float) getDistance(processedInstructions.get(processedInstructions.size() - 1).cornerLocation, processedInstruction.cornerLocation) * mmPerPixel / 1000;
      }
      Float distance = distanceMeters;
      String units = "";

      switch(unit) {
        case "FEET":
          distance *= 3.28084F;
          units = Math.ceil(distance) == 1 ? " foot" : " feet";
          break;
        case "METERS":
        default:
          units = Math.ceil(distance) == 1 ? " meter" : " meters";
          break;
      }
      processedInstruction.distance = Double.valueOf(distanceMeters);
      processedInstruction.remainingDistance = Double.valueOf(distanceMeters);
      processedInstruction.direction = instruction.getDirection();
      if (instruction.getText().startsWith("Arrive")) {
        processedInstruction.direction = "Arrive";
      }

      String prepend = "";
      if (instruction.getDirection().equals("Left") || instruction.getDirection().equals("Right")) {
        prepend = "Turn ";
      }
      processedInstruction.prepend = prepend;

      if (instruction.getSurroundingElements() != null &&
        instruction.getSurroundingElements().getShapes() != null &&
        instruction.getSurroundingElements().getShapes().length > 0) {
        JShapeDrawable shape = instruction.getSurroundingElements().getShapes()[0].getShape();

        //Get data-unit meta data which is associated to units
        Object[] dataUnits = shape.getMetaData("data-unit");

        //Make sure shape has destination association
        if (dataUnits != null && dataUnits.length > 0) {
          int destinationId = (int) dataUnits[0];

          //Get destination object from Id
          Destination destination = venue.getDestinations().getById(destinationId);

          //Get destination name
          String destinationName = destination.getName();

          if (destinationName != null) {
            processedInstruction.destinationName = destinationName;

            processedInstruction.dest = destination.id;
            processedInstruction.fullText = instruction.getText() + " at " + destinationName + " in " + (int)Math.ceil(distance) + units;

          } else {
            processedInstruction.dest = destination.id;
            processedInstruction.fullText = instruction.getText() + " in " + (int)Math.ceil(distance) + units;
          }
        } else {
          processedInstruction.destinationName = "";
          processedInstruction.dest = null;
          processedInstruction.fullText = instruction.getText() + " in " + (int)Math.ceil(distance) + units;
        }
      } else {
        processedInstruction.destinationName = "";
        processedInstruction.dest = null;
        processedInstruction.fullText = instruction.getText() + " in " + (int)Math.ceil(distance) + units;
      }
      processedInstructions.add(processedInstruction);
    }

    return processedInstructions.toArray(new ProcessedInstruction[processedInstructions.size()]);
  }

  public static WritableArray getMapFromPath(JController controller, PathPerFloor[] pathPerFloors) {
    WritableArray pathArr = ReactUtilities.classToArray(pathPerFloors);
    WritableArray pathArrW = Arguments.createArray();
    for (int i = 0; i < pathArr.size(); i++) {
      ReadableMap path = pathArr.getMap(i);

      WritableMap pathW = Arguments.createMap();
      pathW.putInt("mapId", path.getInt("mapId"));
      pathW.putMap("mover", path.getMap("mover"));
      WritableArray pointsW = Arguments.createArray();
      pathW.putDouble("cost", path.getDouble("cost"));
      pathW.putDouble("pixelDistance", path.getDouble("pixelDistance"));
      pathW.putDouble("mmDistance", path.getDouble("mmDistance"));

      for (int j = 0; j < path.getArray("points").size(); j++) {

        ReadableMap point = path.getArray("points").getMap(j);
        WritableMap pointW = Arguments.createMap();

        pointW.putInt("id", point.getInt("id"));
        pointW.putInt("mapId", point.getInt("mapId"));
        pointW.putDouble("x", point.getDouble("x"));
        pointW.putDouble("y", point.getDouble("y"));
        pointW.putDouble("z", point.getDouble("z"));

        PointF latLng = convertLocalToLatLng(controller, (float)point.getDouble("x"), (float)point.getDouble("y"));
        pointW.putDouble("latitude", latLng.x);
        pointW.putDouble("longitude", latLng.y);

        int id = point.getInt("id");
        Waypoint waypoint = Arrays.stream(controller.getActiveVenue().getMaps().getAllWaypoints()).filter(wp -> wp.id == id).findFirst().get();
        Associations associations = waypoint.getAssociations();
        WritableMap assocaitionsMap = ReactUtilities.classToMap(associations);

        pointW.putMap("associations", assocaitionsMap);
        pointsW.pushMap(pointW);
      }

      pathW.putArray("points", pointsW);
      pathArrW.pushMap(pathW);
    }
    return pathArrW;
  }
}
